/*---------------------------------------------------------------------------
  otp a simple one time pad encrytion program.
  Copyright (C) B. Davis 2023, 2024
  <ben@x86toapple.org>, <ben@xspsoft.org>
-----------------------------------------------------------------------------*/

#define SUPPORT_KEYREWIND 0 /* Enable(1), or Disable(0) keyfile rewinding */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>


void usage ();

int main (int argc, char *argv[]) {

  int key = 1;     /* character read from pad file. */
  int char1 = 1;   /* character from input file, which is read, changed, */
                   /* and then written to output file. */
  int encrypt = 1; /* if set to 1, encrypts.  if set to 0, decrypts. */

  FILE *infile, *outfile, *padfile;

  if (argc != 5) {
    usage();
    return 0;
  }

  encrypt = strcmp(argv[1], "-d");

  if (chmod(argv[2], 00400) == -1) {
    perror("otp: Could not set input file to readonly");
  }

  infile = fopen(argv[2], "rb");

  if (infile == 0) {
    perror("otp: infile");
    return 2;
  }
  
  outfile = fopen(argv[3], "wb");

  if (outfile == 0) {
    perror("otp: outfile");
    return 2;
  }

  if (chmod(argv[4], 00400) == -1) {
    perror("otp: Could not set padfile to readonly");
  }

  padfile = fopen(argv[4], "rb");
  
  if (padfile == 0 ) {
    perror("otp: padfile");
    return 2;
  }

  char1 = getc(infile);
  key = getc(padfile);

  while (char1 != EOF) {
    if (encrypt == 1) {
      char1 = char1 + key;
    } else {
      char1 = char1 - key;
    }

    putc(char1, outfile);
    char1 = getc(infile);
    key = getc(padfile);

#if SUPPORT_KEYREWIND == 1
    if (key == EOF) {
      rewind(padfile);
      key = getc(padfile);
    }
#else
   if (key == EOF) {
     fprintf(stderr, "End of keyfile. Exiting...\n");
     fclose(infile);
     fclose(outfile);
     fclose(padfile);
     return 2;
   }
#endif

  }

  fclose(infile);
  fclose(outfile);
  fclose(padfile);
  return 0;
}

void usage () {
  printf("Usage:\n");
  printf("otp [-e|-d] infile outfile padfile\n");
}
